<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Controller;

use Sylius\Bundle\ResourceBundle\Controller\AuthorizationCheckerInterface;
use Sylius\Bundle\ResourceBundle\Controller\RequestConfiguration;
use Sylius\Component\Core\Model\AdminUserInterface;
use Sylius\Component\Customer\Context\CustomerContextInterface;
use Symfony\Component\Security\Core\Security;

class QuoteAuthorizationChecker implements AuthorizationCheckerInterface
{
    private Security $security;

    private AuthorizationCheckerInterface $decoratedAuthorizationChecker;

    private CustomerContextInterface $customerContext;

    public function __construct(
        Security $security,
        AuthorizationCheckerInterface $decoratedAuthorizationChecker,
        CustomerContextInterface $customerContext
    ) {
        $this->security = $security;
        $this->decoratedAuthorizationChecker = $decoratedAuthorizationChecker;
        $this->customerContext = $customerContext;
    }

    public function isGranted(RequestConfiguration $configuration, string $permission): bool
    {
        if ($this->security->getUser() instanceof AdminUserInterface) {
            return true;
        }

        if ('dedi_quote.quote.update' === $permission) {
            return $this->customerIsGranted();
        }

        return $this->decoratedAuthorizationChecker->isGranted($configuration, $permission);
    }

    private function customerIsGranted(): bool
    {
        return null !== $this->customerContext->getCustomer();
    }
}
